// gp_d3d9.cpp 

#include "stdafx.h"
#include "resource.h"
#include "gp_d3d9.h"

// global variables
#pragma data_seg (".d3d9_shared")
myIDirect3DDevice9*   gl_pmyIDirect3DDevice9;
myIDirect3D9*         gl_pmyIDirect3D9;
myIDirect3DDevice9Ex* gl_pmyIDirect3DDevice9Ex;
myIDirect3D9Ex*       gl_pmyIDirect3D9Ex;
HINSTANCE             gl_hOriginalDll;
HINSTANCE             gl_hThisInstance;
#pragma data_seg ()

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    // to avoid compiler lvl4 warnings 
    LPVOID lpDummy = lpReserved;
    lpDummy = NULL;
    
    switch (ul_reason_for_call)
	{
	    case DLL_PROCESS_ATTACH: InitInstance(hModule); break;
	    case DLL_PROCESS_DETACH: ExitInstance(); break;
        
        case DLL_THREAD_ATTACH:  break;
	    case DLL_THREAD_DETACH:  break;
	}
    return(true);
}

// Exported function (faking d3d9.dll's one-and-only export)
IDirect3D9* WINAPI Direct3DCreate9(UINT SDKVersion)
{
	DB(L"GPROXY: Exported function Direct3DCreate9 reached.\r\n");
	DB(L"GPROXY: Direct3DCreate9 called with SDKVersion No.", (int)SDKVersion);
	
	if (!gl_hOriginalDll) LoadOriginalDll(); // looking for the "right d3d9.dll"
		
	// Hooking IDirect3D Object from Original Library
	typedef IDirect3D9* (WINAPI* D3D9_Type)(UINT SDKVersion);
	D3D9_Type D3DCreate9_fn = (D3D9_Type) GetProcAddress( gl_hOriginalDll, "Direct3DCreate9");
    
    //Debug
	if (!D3DCreate9_fn) 
    {
        DB(L"GPROXY: Pointer to original D3DCreate9 function not received ERROR ****\r\n");
        ::ExitProcess(0);
    }
	
	// Request pointer from Original Dll. 
	IDirect3D9 *pIDirect3D9_orig = D3DCreate9_fn(SDKVersion);
	
	// Create my IDirect3D9 object and store pointer to original object there.
	// note: the object will delete itself once Ref count is zero (similar to COM objects)
	gl_pmyIDirect3D9 = new myIDirect3D9(pIDirect3D9_orig);
	
	// Return pointer to hooking Object instead of "real one"
	return (gl_pmyIDirect3D9);
}

HRESULT WINAPI Direct3DCreate9Ex(UINT SDKVersion, IDirect3D9Ex** ppDirect3D9Ex)
{
	DB(L"GPROXY: Exported function Direct3DCreate9Ex reached.\r\n");
	DB(L"GPROXY: Direct3DCreate9Ex called with SDKVersion No.", (int)SDKVersion);
	DB(L"\r\n");
	
	if (!gl_hOriginalDll) LoadOriginalDll(); // looking for the "right d3d9.dll"
	
	// Hooking IDirect3D Object from Original Library
	typedef HRESULT (WINAPI* D3D9Ex_Type)(UINT SDKVersion, IDirect3D9Ex **ppD3D);
	D3D9Ex_Type D3DCreate9Ex_fn = (D3D9Ex_Type) GetProcAddress( gl_hOriginalDll, "Direct3DCreate9Ex");
    
    //Debug
	if (!D3DCreate9Ex_fn) 
    {
        DB(L"GPROXY: Pointer to original D3DCreate9Ex function not received ERROR ****\r\n");
        ::ExitProcess(0);
    }
	
	// Request pointer from Original Dll. 
	IDirect3D9Ex *pIDirect3D9Ex_orig;
	HRESULT hres = D3DCreate9Ex_fn(SDKVersion, &pIDirect3D9Ex_orig);

	// If hres is S_OK, create my IDirect3D9Ex object and store pointer to original object there.
	// note: the object will delete itself once Ref count is zero (similar to COM objects)
	if (hres == S_OK)
	{
		DB(L"GPROXY: D3DCreate9Ex_fn succeeded\r\n");
		gl_pmyIDirect3D9Ex = new myIDirect3D9Ex(pIDirect3D9Ex_orig);
		*ppDirect3D9Ex = gl_pmyIDirect3D9Ex;
		return(S_OK);
	}

	// Something went wrong. Pass back original values, not creating proxy object
	DB(L"GPROXY: D3DCreate9Ex_fn failed ERROR ****\r\n");
	*ppDirect3D9Ex = pIDirect3D9Ex_orig;
	return(hres);
}

void InitInstance(HANDLE hModule) 
{
#if defined _M_IX86
	DB(L"GPROXY: InitInstance. Version ", (float)GP_DLLVERSION);
#else
	DB(L"GPROXY: InitInstance. Version (64bit)  ", (float)GP_DLLVERSION);
#endif	

	// Initialisation
	gl_hOriginalDll          = NULL;
	gl_hThisInstance         = NULL;
	gl_pmyIDirect3D9         = NULL;
	gl_pmyIDirect3DDevice9   = NULL;
	gl_pmyIDirect3D9Ex       = NULL;
	gl_pmyIDirect3DDevice9Ex = NULL;
	
	// Storing Instance handle into global var
	gl_hThisInstance = (HINSTANCE)  hModule;

	// Show the startup screen
	ShowStartupScreen();
}

int WINAPI D3DPERF_BeginEvent(D3DCOLOR col, LPCWSTR wszName) // DX9c
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef int (WINAPI* D3D9_Type)(D3DCOLOR col, LPCWSTR wszName);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "D3DPERF_BeginEvent");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original D3DPERF_BeginEvent function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	return(D3D9_fn(col, wszName));
}

int WINAPI D3DPERF_EndEvent(void)// DX9c
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef int (WINAPI* D3D9_Type)(void);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "D3DPERF_EndEvent");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original D3DPERF_EndEvent function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	return(D3D9_fn());
}

void WINAPI D3DPERF_SetMarker(D3DCOLOR col, LPCWSTR wszName)// DX9c
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef void (WINAPI* D3D9_Type)(D3DCOLOR col, LPCWSTR wszName);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "D3DPERF_SetMarker");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original D3DPERF_SetMarker function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	D3D9_fn(col, wszName);
}

void WINAPI D3DPERF_SetRegion(D3DCOLOR col, LPCWSTR wszName)// DX9c
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef void (WINAPI* D3D9_Type)(D3DCOLOR col, LPCWSTR wszName);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "D3DPERF_SetRegion");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original D3DPERF_SetRegion function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	D3D9_fn(col, wszName);
}

BOOL WINAPI D3DPERF_QueryRepeatFrame(void)// DX9c
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef BOOL (WINAPI* D3D9_Type)(void);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "D3DPERF_QueryRepeatFrame");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original D3DPERF_QueryRepeatFrame function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	return(D3D9_fn());
}

void WINAPI D3DPERF_SetOptions(DWORD dwOptions)// DX9c
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef void (WINAPI* D3D9_Type)(DWORD dwOption);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "D3DPERF_SetOptions");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original D3DPERF_SetOptions function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	D3D9_fn(dwOptions);
}

DWORD WINAPI D3DPERF_GetStatus(void)// DX9c
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef DWORD (WINAPI* D3D9_Type)(void);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "D3DPERF_GetStatus");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original D3DPERF_GetStatus function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	return(D3D9_fn());
}

void WINAPI Direct3DShaderValidatorCreate9(void)
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef HRESULT (WINAPI* D3D9_Type)(void);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "Direct3DShaderValidatorCreate9");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original Direct3DShaderValidatorCreate9 function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	D3D9_fn();
}

HRESULT WINAPI PSGPError(void)
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef HRESULT (WINAPI* D3D9_Type)(void);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "PSGPError");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original PSGPError function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	return(D3D9_fn());
}

HRESULT WINAPI PSGPSampleTexture(void)
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef HRESULT (WINAPI* D3D9_Type)(void);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "PSGPSampleTexture");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original PSGPSampleTexture function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	return(D3D9_fn());
}

HRESULT WINAPI DebugSetLevel(void)
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef HRESULT (WINAPI* D3D9_Type)(void);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "DebugSetLevel");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original DebugSetLevel function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	return(D3D9_fn());
}

void WINAPI DebugSetMute(void)
{
	if (!gl_hOriginalDll) LoadOriginalDll();
	
	// Calling routine from original dll
	typedef void (WINAPI* D3D9_Type)(void);
	D3D9_Type D3D9_fn = (D3D9_Type) GetProcAddress(gl_hOriginalDll, "DebugSetMute");

	//Debug
	if (!D3D9_fn) 
	{
		DB(L"GPROXY: Pointer to original DebugSetMute function not received ERROR ****\r\n");
		::ExitProcess(0);
	}

	D3D9_fn();
}

void LoadOriginalDll(void)
{
    wchar_t buffer[MAX_PATH];
    
    // Getting path to system dir and to d3d9.dll
	if (::GetSystemDirectory(buffer, MAX_PATH) > 0)
	{
		// Append dll name
		wcscat_s(buffer, MAX_PATH, L"\\d3d9.dll");

		if (!gl_hOriginalDll) gl_hOriginalDll = ::LoadLibrary(buffer);
	}

	// Debug
	if (!gl_hOriginalDll)
	{
		DB(L"GPROXY: Original d3d9.dll not loaded ERROR ****\r\n");
		::ExitProcess(0); // exit the hard way
	}
}

void ExitInstance() 
{
	DB(L"GPROXY: ExitInstance.\r\n");
    
    if (gl_hOriginalDll)
	{
		::FreeLibrary(gl_hOriginalDll);
	    gl_hOriginalDll = NULL;  
	}
}

void ShowStartupScreen() 
{
	DB(L"GPROXY: Startup Screen.\r\n");

#if defined _M_IX86
	HBITMAP hBM = ::LoadBitmap(gl_hThisInstance, MAKEINTRESOURCE(IDB_GPPACTIVE_BITMAP));
#else
	HBITMAP hBM = ::LoadBitmap(gl_hThisInstance,MAKEINTRESOURCE(IDB_GPPACTIVE_BITMAP64));
#endif	

    if (hBM)
    {
        HDC hDC = ::GetDC(NULL);
        if (hDC)
        {
            int iXPos = (::GetDeviceCaps(hDC,HORZRES)/2) - (128/2);
            int iYPos = (::GetDeviceCaps(hDC,VERTRES)/2) - (128/2);

            // paint the "GPP active" sign on desktop
            HDC hMemDC = ::CreateCompatibleDC(hDC);
            HBITMAP hBMold = (HBITMAP) ::SelectObject(hMemDC,hBM);
            ::BitBlt(hDC,iXPos,iYPos,128,128,hMemDC,0,0,SRCCOPY);

            //Cleanup
            ::SelectObject(hMemDC,hBMold);
            ::DeleteDC(hMemDC);
            ::ReleaseDC(NULL,hDC);

            // Wait 2 seconds before proceeding
            ::Sleep(2000);
        }
        ::DeleteObject(hBM);
    }
}


