#include "StdAfx.h"
#include "myIDirect3DDevice9Ex.h"

myIDirect3DDevice9Ex::myIDirect3DDevice9Ex(IDirect3DDevice9Ex* pOriginal)
{
    m_pIDirect3DDevice9Ex = pOriginal; // store the pointer to original object
    m_pComms    = new CGPComms();
    m_pGraphics = new CGPGraphics(pOriginal, m_pComms);
}

myIDirect3DDevice9Ex::~myIDirect3DDevice9Ex(void)
{
}

HRESULT myIDirect3DDevice9Ex::QueryInterface (REFIID riid, void** ppvObj)
{
    // check if original dll can provide interface. then send *our* address
	*ppvObj = NULL;

	HRESULT hRes = m_pIDirect3DDevice9Ex->QueryInterface(riid, ppvObj); 

	if (hRes == NOERROR)
	{
		*ppvObj = this;
	}
	
	return hRes;
}

ULONG myIDirect3DDevice9Ex::AddRef(void)
{
    return(m_pIDirect3DDevice9Ex->AddRef());
}

ULONG myIDirect3DDevice9Ex::Release(void)
{
    // ATTENTION: This is a booby-trap ! Watch out !
	// As we create our own sprites, surfaces, etc. (thus increasing the ref counter
	// by external action), we need to delete that objects before calling the original
	// Release function
	
	// global var
	extern myIDirect3DDevice9Ex* gl_pmyIDirect3DDevice9Ex;

    // release/delete own objects
    if (m_pGraphics) delete(m_pGraphics);
    if (m_pComms)    delete(m_pComms);
	
	// Calling original function now
	ULONG count = m_pIDirect3DDevice9Ex->Release();
	
	DB(L"GPROXY: DeviceEx Release RetVal=",(int)count); 
	
    // now, the Original Object has deleted itself, so do we here
	gl_pmyIDirect3DDevice9Ex = NULL;
	delete(this);  // destructor will be called automatically

	return (count);
}

HRESULT myIDirect3DDevice9Ex::TestCooperativeLevel(void)
{
    return(m_pIDirect3DDevice9Ex->TestCooperativeLevel());
}

UINT myIDirect3DDevice9Ex::GetAvailableTextureMem(void)
{
    return(m_pIDirect3DDevice9Ex->GetAvailableTextureMem());
}

HRESULT myIDirect3DDevice9Ex::EvictManagedResources(void)
{
    return(m_pIDirect3DDevice9Ex->EvictManagedResources());
}

HRESULT myIDirect3DDevice9Ex::GetDirect3D(IDirect3D9** ppD3D9)
{
    return(m_pIDirect3DDevice9Ex->GetDirect3D(ppD3D9));
}

HRESULT myIDirect3DDevice9Ex::GetDeviceCaps(D3DCAPS9* pCaps)
{
    return(m_pIDirect3DDevice9Ex->GetDeviceCaps(pCaps));
}

HRESULT myIDirect3DDevice9Ex::GetDisplayMode(UINT iSwapChain,D3DDISPLAYMODE* pMode)
{
    return(m_pIDirect3DDevice9Ex->GetDisplayMode(iSwapChain, pMode));
}

HRESULT myIDirect3DDevice9Ex::GetCreationParameters(D3DDEVICE_CREATION_PARAMETERS *pParameters)
{
    return(m_pIDirect3DDevice9Ex->GetCreationParameters(pParameters));
}

HRESULT myIDirect3DDevice9Ex::SetCursorProperties(UINT XHotSpot,UINT YHotSpot,IDirect3DSurface9* pCursorBitmap)
{
    return(m_pIDirect3DDevice9Ex->SetCursorProperties(XHotSpot,YHotSpot,pCursorBitmap));
}

void    myIDirect3DDevice9Ex::SetCursorPosition(int X,int Y,DWORD Flags)
{
    return(m_pIDirect3DDevice9Ex->SetCursorPosition(X,Y,Flags));
}

BOOL    myIDirect3DDevice9Ex::ShowCursor(BOOL bShow)
{
    return(m_pIDirect3DDevice9Ex->ShowCursor(bShow));
}

HRESULT myIDirect3DDevice9Ex::CreateAdditionalSwapChain(D3DPRESENT_PARAMETERS* pPresentationParameters,IDirect3DSwapChain9** pSwapChain)  
{
    return(m_pIDirect3DDevice9Ex->CreateAdditionalSwapChain(pPresentationParameters,pSwapChain));
}

HRESULT myIDirect3DDevice9Ex::GetSwapChain(UINT iSwapChain,IDirect3DSwapChain9** pSwapChain)
{
    return(m_pIDirect3DDevice9Ex->GetSwapChain(iSwapChain,pSwapChain));
}

UINT    myIDirect3DDevice9Ex::GetNumberOfSwapChains(void)
{
    return(m_pIDirect3DDevice9Ex->GetNumberOfSwapChains());
}

HRESULT myIDirect3DDevice9Ex::Reset(D3DPRESENT_PARAMETERS* pPresentationParameters)
{
    return(m_pIDirect3DDevice9Ex->Reset(pPresentationParameters));
}

HRESULT myIDirect3DDevice9Ex::Present(CONST RECT* pSourceRect,CONST RECT* pDestRect,HWND hDestWindowOverride,CONST RGNDATA* pDirtyRegion)
{
    // draw our own things on top
    m_pGraphics->Render();
    
    // call original routine
	HRESULT hres = m_pIDirect3DDevice9Ex->Present( pSourceRect, pDestRect, hDestWindowOverride, pDirtyRegion);

	return (hres);
}

HRESULT myIDirect3DDevice9Ex::GetBackBuffer(UINT iSwapChain,UINT iBackBuffer,D3DBACKBUFFER_TYPE Type,IDirect3DSurface9** ppBackBuffer)
{
    return(m_pIDirect3DDevice9Ex->GetBackBuffer(iSwapChain,iBackBuffer,Type,ppBackBuffer));
}

HRESULT myIDirect3DDevice9Ex::GetRasterStatus(UINT iSwapChain,D3DRASTER_STATUS* pRasterStatus)
{
    return(m_pIDirect3DDevice9Ex->GetRasterStatus(iSwapChain,pRasterStatus));
}

HRESULT myIDirect3DDevice9Ex::SetDialogBoxMode(BOOL bEnableDialogs)
{
    return(m_pIDirect3DDevice9Ex->SetDialogBoxMode(bEnableDialogs));
}

void    myIDirect3DDevice9Ex::SetGammaRamp(UINT iSwapChain,DWORD Flags,CONST D3DGAMMARAMP* pRamp)
{
    return(m_pIDirect3DDevice9Ex->SetGammaRamp(iSwapChain,Flags,pRamp));
}

void    myIDirect3DDevice9Ex::GetGammaRamp(UINT iSwapChain,D3DGAMMARAMP* pRamp)
{
    return(m_pIDirect3DDevice9Ex->GetGammaRamp(iSwapChain,pRamp));
}

HRESULT myIDirect3DDevice9Ex::CreateTexture(UINT Width,UINT Height,UINT Levels,DWORD Usage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DTexture9** ppTexture,HANDLE* pSharedHandle)
{
    return(m_pIDirect3DDevice9Ex->CreateTexture(Width,Height,Levels,Usage,Format,Pool,ppTexture,pSharedHandle));
}

HRESULT myIDirect3DDevice9Ex::CreateVolumeTexture(UINT Width,UINT Height,UINT Depth,UINT Levels,DWORD Usage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DVolumeTexture9** ppVolumeTexture,HANDLE* pSharedHandle)
{
    return(m_pIDirect3DDevice9Ex->CreateVolumeTexture(Width,Height,Depth,Levels,Usage,Format,Pool,ppVolumeTexture,pSharedHandle));
}

HRESULT myIDirect3DDevice9Ex::CreateCubeTexture(UINT EdgeLength,UINT Levels,DWORD Usage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DCubeTexture9** ppCubeTexture,HANDLE* pSharedHandle)
{
    return(m_pIDirect3DDevice9Ex->CreateCubeTexture(EdgeLength,Levels,Usage,Format,Pool,ppCubeTexture,pSharedHandle));
}

HRESULT myIDirect3DDevice9Ex::CreateVertexBuffer(UINT Length,DWORD Usage,DWORD FVF,D3DPOOL Pool,IDirect3DVertexBuffer9** ppVertexBuffer,HANDLE* pSharedHandle)
{
    return(m_pIDirect3DDevice9Ex->CreateVertexBuffer(Length,Usage,FVF,Pool,ppVertexBuffer,pSharedHandle));
}

HRESULT myIDirect3DDevice9Ex::CreateIndexBuffer(UINT Length,DWORD Usage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DIndexBuffer9** ppIndexBuffer,HANDLE* pSharedHandle)
{
    return(m_pIDirect3DDevice9Ex->CreateIndexBuffer(Length,Usage,Format,Pool,ppIndexBuffer,pSharedHandle));
}

HRESULT myIDirect3DDevice9Ex::CreateRenderTarget(UINT Width,UINT Height,D3DFORMAT Format,D3DMULTISAMPLE_TYPE MultiSample,DWORD MultisampleQuality,BOOL Lockable,IDirect3DSurface9** ppSurface,HANDLE* pSharedHandle)
{
    return(m_pIDirect3DDevice9Ex->CreateRenderTarget(Width,Height,Format,MultiSample,MultisampleQuality,Lockable,ppSurface,pSharedHandle));
}

HRESULT myIDirect3DDevice9Ex::CreateDepthStencilSurface(UINT Width,UINT Height,D3DFORMAT Format,D3DMULTISAMPLE_TYPE MultiSample,DWORD MultisampleQuality,BOOL Discard,IDirect3DSurface9** ppSurface,HANDLE* pSharedHandle)
{
    return(m_pIDirect3DDevice9Ex->CreateDepthStencilSurface(Width,Height,Format,MultiSample,MultisampleQuality,Discard,ppSurface,pSharedHandle));
}

HRESULT myIDirect3DDevice9Ex::UpdateSurface(IDirect3DSurface9* pSourceSurface,CONST RECT* pSourceRect,IDirect3DSurface9* pDestinationSurface,CONST POINT* pDestPoint)
{
    return(m_pIDirect3DDevice9Ex->UpdateSurface(pSourceSurface,pSourceRect,pDestinationSurface,pDestPoint));
}

HRESULT myIDirect3DDevice9Ex::UpdateTexture(IDirect3DBaseTexture9* pSourceTexture,IDirect3DBaseTexture9* pDestinationTexture)
{
    return(m_pIDirect3DDevice9Ex->UpdateTexture(pSourceTexture,pDestinationTexture));
}

HRESULT myIDirect3DDevice9Ex::GetRenderTargetData(IDirect3DSurface9* pRenderTarget,IDirect3DSurface9* pDestSurface)
{
    return(m_pIDirect3DDevice9Ex->GetRenderTargetData(pRenderTarget,pDestSurface));
}

HRESULT myIDirect3DDevice9Ex::GetFrontBufferData(UINT iSwapChain,IDirect3DSurface9* pDestSurface)
{
    return(m_pIDirect3DDevice9Ex->GetFrontBufferData(iSwapChain,pDestSurface));
}

HRESULT myIDirect3DDevice9Ex::StretchRect(IDirect3DSurface9* pSourceSurface,CONST RECT* pSourceRect,IDirect3DSurface9* pDestSurface,CONST RECT* pDestRect,D3DTEXTUREFILTERTYPE Filter)
{
    return(m_pIDirect3DDevice9Ex->StretchRect(pSourceSurface,pSourceRect,pDestSurface,pDestRect,Filter));
}

HRESULT myIDirect3DDevice9Ex::ColorFill(IDirect3DSurface9* pSurface,CONST RECT* pRect,D3DCOLOR color)
{
    return(m_pIDirect3DDevice9Ex->ColorFill(pSurface,pRect,color));
}

HRESULT myIDirect3DDevice9Ex::CreateOffscreenPlainSurface(UINT Width,UINT Height,D3DFORMAT Format,D3DPOOL Pool,IDirect3DSurface9** ppSurface,HANDLE* pSharedHandle)
{
    return(m_pIDirect3DDevice9Ex->CreateOffscreenPlainSurface(Width,Height,Format,Pool,ppSurface,pSharedHandle));
}

HRESULT myIDirect3DDevice9Ex::SetRenderTarget(DWORD RenderTargetIndex,IDirect3DSurface9* pRenderTarget)
{
    return(m_pIDirect3DDevice9Ex->SetRenderTarget(RenderTargetIndex,pRenderTarget));
}

HRESULT myIDirect3DDevice9Ex::GetRenderTarget(DWORD RenderTargetIndex,IDirect3DSurface9** ppRenderTarget)
{
    return(m_pIDirect3DDevice9Ex->GetRenderTarget(RenderTargetIndex,ppRenderTarget));
}

HRESULT myIDirect3DDevice9Ex::SetDepthStencilSurface(IDirect3DSurface9* pNewZStencil)
{
    return(m_pIDirect3DDevice9Ex->SetDepthStencilSurface(pNewZStencil));
}

HRESULT myIDirect3DDevice9Ex::GetDepthStencilSurface(IDirect3DSurface9** ppZStencilSurface)
{
    return(m_pIDirect3DDevice9Ex->GetDepthStencilSurface(ppZStencilSurface));
}

HRESULT myIDirect3DDevice9Ex::BeginScene(void)
{
    return(m_pIDirect3DDevice9Ex->BeginScene());
}

HRESULT myIDirect3DDevice9Ex::EndScene(void)
{
    return(m_pIDirect3DDevice9Ex->EndScene());
}

HRESULT myIDirect3DDevice9Ex::Clear(DWORD Count,CONST D3DRECT* pRects,DWORD Flags,D3DCOLOR Color,float Z,DWORD Stencil)
{
    return(m_pIDirect3DDevice9Ex->Clear(Count,pRects,Flags,Color,Z,Stencil));
}

HRESULT myIDirect3DDevice9Ex::SetTransform(D3DTRANSFORMSTATETYPE State,CONST D3DMATRIX* pMatrix)
{
    return(m_pIDirect3DDevice9Ex->SetTransform(State,pMatrix));
}

HRESULT myIDirect3DDevice9Ex::GetTransform(D3DTRANSFORMSTATETYPE State,D3DMATRIX* pMatrix)
{
    return(m_pIDirect3DDevice9Ex->GetTransform(State,pMatrix));
}

HRESULT myIDirect3DDevice9Ex::MultiplyTransform(D3DTRANSFORMSTATETYPE State,CONST D3DMATRIX* pMatrix)
{
    return(m_pIDirect3DDevice9Ex->MultiplyTransform(State,pMatrix));
}

HRESULT myIDirect3DDevice9Ex::SetViewport(CONST D3DVIEWPORT9* pViewport)
{
    return(m_pIDirect3DDevice9Ex->SetViewport(pViewport));
}

HRESULT myIDirect3DDevice9Ex::GetViewport(D3DVIEWPORT9* pViewport)
{
    return(m_pIDirect3DDevice9Ex->GetViewport(pViewport));
}

HRESULT myIDirect3DDevice9Ex::SetMaterial(CONST D3DMATERIAL9* pMaterial)
{
    return(m_pIDirect3DDevice9Ex->SetMaterial(pMaterial));
}

HRESULT myIDirect3DDevice9Ex::GetMaterial(D3DMATERIAL9* pMaterial)
{
    return(m_pIDirect3DDevice9Ex->GetMaterial(pMaterial));
}

HRESULT myIDirect3DDevice9Ex::SetLight(DWORD Index,CONST D3DLIGHT9* pLight)
{
    return(m_pIDirect3DDevice9Ex->SetLight(Index,pLight));
}

HRESULT myIDirect3DDevice9Ex::GetLight(DWORD Index,D3DLIGHT9* pLight)
{
    return(m_pIDirect3DDevice9Ex->GetLight(Index,pLight));
}

HRESULT myIDirect3DDevice9Ex::LightEnable(DWORD Index,BOOL Enable)
{
    return(m_pIDirect3DDevice9Ex->LightEnable(Index,Enable));
}

HRESULT myIDirect3DDevice9Ex::GetLightEnable(DWORD Index,BOOL* pEnable)
{
    return(m_pIDirect3DDevice9Ex->GetLightEnable(Index, pEnable));
}

HRESULT myIDirect3DDevice9Ex::SetClipPlane(DWORD Index,CONST float* pPlane)
{
    return(m_pIDirect3DDevice9Ex->SetClipPlane(Index, pPlane));
}

HRESULT myIDirect3DDevice9Ex::GetClipPlane(DWORD Index,float* pPlane)
{
    return(m_pIDirect3DDevice9Ex->GetClipPlane(Index,pPlane));
}

HRESULT myIDirect3DDevice9Ex::SetRenderState(D3DRENDERSTATETYPE State,DWORD Value)
{
    return(m_pIDirect3DDevice9Ex->SetRenderState(State, Value));
}

HRESULT myIDirect3DDevice9Ex::GetRenderState(D3DRENDERSTATETYPE State,DWORD* pValue)
{
    return(m_pIDirect3DDevice9Ex->GetRenderState(State, pValue));
}

HRESULT myIDirect3DDevice9Ex::CreateStateBlock(D3DSTATEBLOCKTYPE Type,IDirect3DStateBlock9** ppSB)
{
    return(m_pIDirect3DDevice9Ex->CreateStateBlock(Type,ppSB));
}

HRESULT myIDirect3DDevice9Ex::BeginStateBlock(void)
{
    return(m_pIDirect3DDevice9Ex->BeginStateBlock());
}

HRESULT myIDirect3DDevice9Ex::EndStateBlock(IDirect3DStateBlock9** ppSB)
{
    return(m_pIDirect3DDevice9Ex->EndStateBlock(ppSB));
}

HRESULT myIDirect3DDevice9Ex::SetClipStatus(CONST D3DCLIPSTATUS9* pClipStatus)
{
    return(m_pIDirect3DDevice9Ex->SetClipStatus(pClipStatus));
}

HRESULT myIDirect3DDevice9Ex::GetClipStatus(D3DCLIPSTATUS9* pClipStatus)
{
    return(m_pIDirect3DDevice9Ex->GetClipStatus( pClipStatus));
}

HRESULT myIDirect3DDevice9Ex::GetTexture(DWORD Stage,IDirect3DBaseTexture9** ppTexture)
{
    return(m_pIDirect3DDevice9Ex->GetTexture(Stage,ppTexture));
}

HRESULT myIDirect3DDevice9Ex::SetTexture(DWORD Stage,IDirect3DBaseTexture9* pTexture)
{
    return(m_pIDirect3DDevice9Ex->SetTexture(Stage,pTexture));
}

HRESULT myIDirect3DDevice9Ex::GetTextureStageState(DWORD Stage,D3DTEXTURESTAGESTATETYPE Type,DWORD* pValue)
{
    return(m_pIDirect3DDevice9Ex->GetTextureStageState(Stage,Type, pValue));
}

HRESULT myIDirect3DDevice9Ex::SetTextureStageState(DWORD Stage,D3DTEXTURESTAGESTATETYPE Type,DWORD Value)
{
    return(m_pIDirect3DDevice9Ex->SetTextureStageState(Stage,Type,Value));
}

HRESULT myIDirect3DDevice9Ex::GetSamplerState(DWORD Sampler,D3DSAMPLERSTATETYPE Type,DWORD* pValue)
{
    return(m_pIDirect3DDevice9Ex->GetSamplerState(Sampler,Type, pValue));
}

HRESULT myIDirect3DDevice9Ex::SetSamplerState(DWORD Sampler,D3DSAMPLERSTATETYPE Type,DWORD Value)
{
    return(m_pIDirect3DDevice9Ex->SetSamplerState(Sampler,Type,Value));
}

HRESULT myIDirect3DDevice9Ex::ValidateDevice(DWORD* pNumPasses)
{
    return(m_pIDirect3DDevice9Ex->ValidateDevice( pNumPasses));
}

HRESULT myIDirect3DDevice9Ex::SetPaletteEntries(UINT PaletteNumber,CONST PALETTEENTRY* pEntries)
{
    return(m_pIDirect3DDevice9Ex->SetPaletteEntries(PaletteNumber, pEntries));
}

HRESULT myIDirect3DDevice9Ex::GetPaletteEntries(UINT PaletteNumber,PALETTEENTRY* pEntries)
{
    return(m_pIDirect3DDevice9Ex->GetPaletteEntries(PaletteNumber, pEntries));
}

HRESULT myIDirect3DDevice9Ex::SetCurrentTexturePalette(UINT PaletteNumber)
{
    return(m_pIDirect3DDevice9Ex->SetCurrentTexturePalette(PaletteNumber));
}

HRESULT myIDirect3DDevice9Ex::GetCurrentTexturePalette(UINT *PaletteNumber)
{
    return(m_pIDirect3DDevice9Ex->GetCurrentTexturePalette(PaletteNumber));
}

HRESULT myIDirect3DDevice9Ex::SetScissorRect(CONST RECT* pRect)
{
    return(m_pIDirect3DDevice9Ex->SetScissorRect( pRect));
}

HRESULT myIDirect3DDevice9Ex::GetScissorRect( RECT* pRect)
{
    return(m_pIDirect3DDevice9Ex->GetScissorRect( pRect));
}

HRESULT myIDirect3DDevice9Ex::SetSoftwareVertexProcessing(BOOL bSoftware)
{
    return(m_pIDirect3DDevice9Ex->SetSoftwareVertexProcessing(bSoftware));
}

BOOL    myIDirect3DDevice9Ex::GetSoftwareVertexProcessing(void)
{
    return(m_pIDirect3DDevice9Ex->GetSoftwareVertexProcessing());
}

HRESULT myIDirect3DDevice9Ex::SetNPatchMode(float nSegments)
{
    return(m_pIDirect3DDevice9Ex->SetNPatchMode(nSegments));
}

float   myIDirect3DDevice9Ex::GetNPatchMode(void)
{
    return(m_pIDirect3DDevice9Ex->GetNPatchMode());
}

HRESULT myIDirect3DDevice9Ex::DrawPrimitive(D3DPRIMITIVETYPE PrimitiveType,UINT StartVertex,UINT PrimitiveCount)
{
    return(m_pIDirect3DDevice9Ex->DrawPrimitive(PrimitiveType,StartVertex,PrimitiveCount));
}

HRESULT myIDirect3DDevice9Ex::DrawIndexedPrimitive(D3DPRIMITIVETYPE PrimitiveType,INT BaseVertexIndex,UINT MinVertexIndex,UINT NumVertices,UINT startIndex,UINT primCount)
{
    return(m_pIDirect3DDevice9Ex->DrawIndexedPrimitive(PrimitiveType,BaseVertexIndex,MinVertexIndex,NumVertices,startIndex,primCount));
}

HRESULT myIDirect3DDevice9Ex::DrawPrimitiveUP(D3DPRIMITIVETYPE PrimitiveType,UINT PrimitiveCount,CONST void* pVertexStreamZeroData,UINT VertexStreamZeroStride)
{
    return(m_pIDirect3DDevice9Ex->DrawPrimitiveUP(PrimitiveType,PrimitiveCount,pVertexStreamZeroData,VertexStreamZeroStride));
}

HRESULT myIDirect3DDevice9Ex::DrawIndexedPrimitiveUP(D3DPRIMITIVETYPE PrimitiveType,UINT MinVertexIndex,UINT NumVertices,UINT PrimitiveCount,CONST void* pIndexData,D3DFORMAT IndexDataFormat,CONST void* pVertexStreamZeroData,UINT VertexStreamZeroStride)
{
    return(m_pIDirect3DDevice9Ex->DrawIndexedPrimitiveUP(PrimitiveType,MinVertexIndex,NumVertices,PrimitiveCount, pIndexData, IndexDataFormat, pVertexStreamZeroData,VertexStreamZeroStride));
}

HRESULT myIDirect3DDevice9Ex::ProcessVertices(UINT SrcStartIndex,UINT DestIndex,UINT VertexCount,IDirect3DVertexBuffer9* pDestBuffer,IDirect3DVertexDeclaration9* pVertexDecl,DWORD Flags)
{
    return(m_pIDirect3DDevice9Ex->ProcessVertices( SrcStartIndex, DestIndex, VertexCount, pDestBuffer, pVertexDecl, Flags));
}

HRESULT myIDirect3DDevice9Ex::CreateVertexDeclaration(CONST D3DVERTEXELEMENT9* pVertexElements,IDirect3DVertexDeclaration9** ppDecl)
{
    return(m_pIDirect3DDevice9Ex->CreateVertexDeclaration( pVertexElements,ppDecl));
}

HRESULT myIDirect3DDevice9Ex::SetVertexDeclaration(IDirect3DVertexDeclaration9* pDecl)
{
    return(m_pIDirect3DDevice9Ex->SetVertexDeclaration(pDecl));
}

HRESULT myIDirect3DDevice9Ex::GetVertexDeclaration(IDirect3DVertexDeclaration9** ppDecl)
{
    return(m_pIDirect3DDevice9Ex->GetVertexDeclaration(ppDecl));
}

HRESULT myIDirect3DDevice9Ex::SetFVF(DWORD FVF)
{
    return(m_pIDirect3DDevice9Ex->SetFVF(FVF));
}

HRESULT myIDirect3DDevice9Ex::GetFVF(DWORD* pFVF)
{
    return(m_pIDirect3DDevice9Ex->GetFVF(pFVF));
}

HRESULT myIDirect3DDevice9Ex::CreateVertexShader(CONST DWORD* pFunction,IDirect3DVertexShader9** ppShader)
{
    return(m_pIDirect3DDevice9Ex->CreateVertexShader(pFunction,ppShader));
}

HRESULT myIDirect3DDevice9Ex::SetVertexShader(IDirect3DVertexShader9* pShader)
{
    return(m_pIDirect3DDevice9Ex->SetVertexShader(pShader));
}

HRESULT myIDirect3DDevice9Ex::GetVertexShader(IDirect3DVertexShader9** ppShader)
{
    return(m_pIDirect3DDevice9Ex->GetVertexShader(ppShader));
}

HRESULT myIDirect3DDevice9Ex::SetVertexShaderConstantF(UINT StartRegister,CONST float* pConstantData,UINT Vector4fCount)
{
    return(m_pIDirect3DDevice9Ex->SetVertexShaderConstantF(StartRegister,pConstantData, Vector4fCount));
}

HRESULT myIDirect3DDevice9Ex::GetVertexShaderConstantF(UINT StartRegister,float* pConstantData,UINT Vector4fCount)
{
    return(m_pIDirect3DDevice9Ex->GetVertexShaderConstantF(StartRegister,pConstantData,Vector4fCount));
}

HRESULT myIDirect3DDevice9Ex::SetVertexShaderConstantI(UINT StartRegister,CONST int* pConstantData,UINT Vector4iCount)
{
    return(m_pIDirect3DDevice9Ex->SetVertexShaderConstantI(StartRegister,pConstantData,Vector4iCount));
}

HRESULT myIDirect3DDevice9Ex::GetVertexShaderConstantI(UINT StartRegister,int* pConstantData,UINT Vector4iCount)
{
    return(m_pIDirect3DDevice9Ex->GetVertexShaderConstantI(StartRegister,pConstantData,Vector4iCount));
}

HRESULT myIDirect3DDevice9Ex::SetVertexShaderConstantB(UINT StartRegister,CONST BOOL* pConstantData,UINT  BoolCount)
{
    return(m_pIDirect3DDevice9Ex->SetVertexShaderConstantB(StartRegister,pConstantData,BoolCount));
}

HRESULT myIDirect3DDevice9Ex::GetVertexShaderConstantB(UINT StartRegister,BOOL* pConstantData,UINT BoolCount)
{
    return(m_pIDirect3DDevice9Ex->GetVertexShaderConstantB(StartRegister,pConstantData,BoolCount));
}

HRESULT myIDirect3DDevice9Ex::SetStreamSource(UINT StreamNumber,IDirect3DVertexBuffer9* pStreamData,UINT OffsetInBytes,UINT Stride)
{
    return(m_pIDirect3DDevice9Ex->SetStreamSource(StreamNumber,pStreamData,OffsetInBytes,Stride));
}

HRESULT myIDirect3DDevice9Ex::GetStreamSource(UINT StreamNumber,IDirect3DVertexBuffer9** ppStreamData,UINT* OffsetInBytes,UINT* pStride)
{
    return(m_pIDirect3DDevice9Ex->GetStreamSource(StreamNumber,ppStreamData,OffsetInBytes,pStride));
}

HRESULT myIDirect3DDevice9Ex::SetStreamSourceFreq(UINT StreamNumber,UINT Divider)
{
    return(m_pIDirect3DDevice9Ex->SetStreamSourceFreq(StreamNumber,Divider));
}

HRESULT myIDirect3DDevice9Ex::GetStreamSourceFreq(UINT StreamNumber,UINT* Divider)
{
    return(m_pIDirect3DDevice9Ex->GetStreamSourceFreq(StreamNumber,Divider));
}

HRESULT myIDirect3DDevice9Ex::SetIndices(IDirect3DIndexBuffer9* pIndexData)
{
    return(m_pIDirect3DDevice9Ex->SetIndices(pIndexData));
}

HRESULT myIDirect3DDevice9Ex::GetIndices(IDirect3DIndexBuffer9** ppIndexData)
{
    return(m_pIDirect3DDevice9Ex->GetIndices(ppIndexData));
}

HRESULT myIDirect3DDevice9Ex::CreatePixelShader(CONST DWORD* pFunction,IDirect3DPixelShader9** ppShader)
{
    return(m_pIDirect3DDevice9Ex->CreatePixelShader(pFunction,ppShader));
}

HRESULT myIDirect3DDevice9Ex::SetPixelShader(IDirect3DPixelShader9* pShader)
{
    return(m_pIDirect3DDevice9Ex->SetPixelShader(pShader));
}

HRESULT myIDirect3DDevice9Ex::GetPixelShader(IDirect3DPixelShader9** ppShader)
{
    return(m_pIDirect3DDevice9Ex->GetPixelShader(ppShader));
}

HRESULT myIDirect3DDevice9Ex::SetPixelShaderConstantF(UINT StartRegister,CONST float* pConstantData,UINT Vector4fCount)
{
    return(m_pIDirect3DDevice9Ex->SetPixelShaderConstantF(StartRegister,pConstantData,Vector4fCount));
}

HRESULT myIDirect3DDevice9Ex::GetPixelShaderConstantF(UINT StartRegister,float* pConstantData,UINT Vector4fCount)
{
    return(m_pIDirect3DDevice9Ex->GetPixelShaderConstantF(StartRegister,pConstantData,Vector4fCount));
}

HRESULT myIDirect3DDevice9Ex::SetPixelShaderConstantI(UINT StartRegister,CONST int* pConstantData,UINT Vector4iCount)
{
    return(m_pIDirect3DDevice9Ex->SetPixelShaderConstantI(StartRegister,pConstantData,Vector4iCount));
}

HRESULT myIDirect3DDevice9Ex::GetPixelShaderConstantI(UINT StartRegister,int* pConstantData,UINT Vector4iCount)
{
    return(m_pIDirect3DDevice9Ex->GetPixelShaderConstantI(StartRegister,pConstantData,Vector4iCount));
}

HRESULT myIDirect3DDevice9Ex::SetPixelShaderConstantB(UINT StartRegister,CONST BOOL* pConstantData,UINT  BoolCount)
{
    return(m_pIDirect3DDevice9Ex->SetPixelShaderConstantB(StartRegister,pConstantData,BoolCount));
}

HRESULT myIDirect3DDevice9Ex::GetPixelShaderConstantB(UINT StartRegister,BOOL* pConstantData,UINT BoolCount)
{
    return(m_pIDirect3DDevice9Ex->GetPixelShaderConstantB(StartRegister,pConstantData,BoolCount));
}

HRESULT myIDirect3DDevice9Ex::DrawRectPatch(UINT Handle,CONST float* pNumSegs,CONST D3DRECTPATCH_INFO* pRectPatchInfo)
{
    return(m_pIDirect3DDevice9Ex->DrawRectPatch(Handle,pNumSegs, pRectPatchInfo));
}

HRESULT myIDirect3DDevice9Ex::DrawTriPatch(UINT Handle,CONST float* pNumSegs,CONST D3DTRIPATCH_INFO* pTriPatchInfo)
{
    return(m_pIDirect3DDevice9Ex->DrawTriPatch(Handle, pNumSegs, pTriPatchInfo));
}

HRESULT myIDirect3DDevice9Ex::DeletePatch(UINT Handle)
{
    return(m_pIDirect3DDevice9Ex->DeletePatch(Handle));
}

HRESULT myIDirect3DDevice9Ex::CreateQuery(D3DQUERYTYPE Type,IDirect3DQuery9** ppQuery)
{
    return(m_pIDirect3DDevice9Ex->CreateQuery(Type,ppQuery));
}

HRESULT __stdcall myIDirect3DDevice9Ex::SetConvolutionMonoKernel(UINT width,UINT height,float* rows,float* columns)
{
    return(m_pIDirect3DDevice9Ex->SetConvolutionMonoKernel(width, height, rows, columns));
}

HRESULT __stdcall myIDirect3DDevice9Ex::ComposeRects(IDirect3DSurface9* pSrc,IDirect3DSurface9* pDst,IDirect3DVertexBuffer9* pSrcRectDescs,UINT NumRects,IDirect3DVertexBuffer9* pDstRectDescs,D3DCOMPOSERECTSOP Operation,int Xoffset,int Yoffset)
{
    return(m_pIDirect3DDevice9Ex->ComposeRects( pSrc, pDst, pSrcRectDescs, NumRects, pDstRectDescs, Operation, Xoffset, Yoffset));
}

HRESULT __stdcall myIDirect3DDevice9Ex::PresentEx(CONST RECT* pSourceRect,CONST RECT* pDestRect,HWND hDestWindowOverride,CONST RGNDATA* pDirtyRegion,DWORD dwFlags)
{
	// draw our own things on top
    m_pGraphics->Render();
    
    // call original routine
	HRESULT hres = m_pIDirect3DDevice9Ex->PresentEx( pSourceRect, pDestRect, hDestWindowOverride, pDirtyRegion, dwFlags);

	return (hres);
}

HRESULT __stdcall myIDirect3DDevice9Ex::GetGPUThreadPriority(INT* pPriority)
{
    return(m_pIDirect3DDevice9Ex->GetGPUThreadPriority(pPriority));
}

HRESULT __stdcall myIDirect3DDevice9Ex::SetGPUThreadPriority(INT Priority)
{
    return(m_pIDirect3DDevice9Ex->SetGPUThreadPriority(Priority));
}

HRESULT __stdcall myIDirect3DDevice9Ex::WaitForVBlank(UINT iSwapChain)
{
    return(m_pIDirect3DDevice9Ex->WaitForVBlank(iSwapChain));
}

HRESULT __stdcall myIDirect3DDevice9Ex::CheckResourceResidency(IDirect3DResource9** pResourceArray,UINT32 NumResources)
{
    return(m_pIDirect3DDevice9Ex->CheckResourceResidency(pResourceArray, NumResources));
}

HRESULT __stdcall myIDirect3DDevice9Ex::SetMaximumFrameLatency(UINT MaxLatency)
{
    return(m_pIDirect3DDevice9Ex->SetMaximumFrameLatency(MaxLatency));
}

HRESULT __stdcall myIDirect3DDevice9Ex::GetMaximumFrameLatency(UINT* pMaxLatency)
{
    return(m_pIDirect3DDevice9Ex->GetMaximumFrameLatency(pMaxLatency));
}

HRESULT __stdcall myIDirect3DDevice9Ex::CheckDeviceState(HWND hDestinationWindow)
{
    return(m_pIDirect3DDevice9Ex->CheckDeviceState(hDestinationWindow));
}

HRESULT __stdcall myIDirect3DDevice9Ex::CreateRenderTargetEx(UINT Width,UINT Height,D3DFORMAT Format,D3DMULTISAMPLE_TYPE MultiSample,DWORD MultisampleQuality,BOOL Lockable,IDirect3DSurface9** ppSurface,HANDLE* pSharedHandle,DWORD Usage)
{
    return(m_pIDirect3DDevice9Ex->CreateRenderTargetEx(Width,Height,Format,MultiSample,MultisampleQuality,Lockable,ppSurface,pSharedHandle,Usage));
}

HRESULT __stdcall myIDirect3DDevice9Ex::CreateOffscreenPlainSurfaceEx(UINT Width,UINT Height,D3DFORMAT Format,D3DPOOL Pool,IDirect3DSurface9** ppSurface,HANDLE* pSharedHandle,DWORD Usage)
{
    return(m_pIDirect3DDevice9Ex->CreateOffscreenPlainSurfaceEx(Width,Height,Format,Pool,ppSurface,pSharedHandle,Usage));
}

HRESULT __stdcall myIDirect3DDevice9Ex::CreateDepthStencilSurfaceEx(UINT Width,UINT Height,D3DFORMAT Format,D3DMULTISAMPLE_TYPE MultiSample,DWORD MultisampleQuality,BOOL Discard,IDirect3DSurface9** ppSurface,HANDLE* pSharedHandle,DWORD Usage)
{
    return(m_pIDirect3DDevice9Ex->CreateDepthStencilSurfaceEx(Width,Height,Format,MultiSample,MultisampleQuality,Discard,ppSurface,pSharedHandle,Usage));
}

HRESULT __stdcall myIDirect3DDevice9Ex::ResetEx(D3DPRESENT_PARAMETERS* pPresentationParameters,D3DDISPLAYMODEEX *pFullscreenDisplayMode)
{
    return(m_pIDirect3DDevice9Ex->ResetEx(pPresentationParameters,pFullscreenDisplayMode));
}

HRESULT __stdcall myIDirect3DDevice9Ex::GetDisplayModeEx(UINT iSwapChain,D3DDISPLAYMODEEX* pMode,D3DDISPLAYROTATION* pRotation)
{
    return(m_pIDirect3DDevice9Ex->GetDisplayModeEx(iSwapChain,pMode,pRotation));
}

